@@ -370,9 +370,20 @@ sub _set_cgo_flags {
     my $bf = Dpkg::BuildFlags->new();
     $bf->load_config();
 
+    if ({$bf->get_features("optimize")}->{"lto"}) {
+        # cgo doesn't support LTO #1013102, https://golang.org/issues/43505
+        warning("LTO opimize is enable in buildflags. But cgo doesn't support it. " .
+                "LTO flags will be stripped in cgo.");
+    }
+
     my @flags = ( "CFLAGS", "CPPFLAGS", "CXXFLAGS", "FFLAGS", "LDFLAGS" );
     foreach my $flag (@flags) {
         if (! exists $ENV{"CGO_" . $flag}) {
+            $bf->strip($flag, "-ffat-lto-objects -flto=auto");
+            if (dpkg_architecture_value("DEB_HOST_ARCH") eq "i386") {
+                # https://github.com/golang/go/issues/54313
+                $bf->strip($flag, "-fstack-protector-strong");
+            }
             $ENV{"CGO_" . $flag} = $bf->get($flag);
         }
     }
@@ -431,6 +442,14 @@ sub _set_gocross {
 }
 
 my ($_go1_minor) = (qx(go version) =~ /go version go1\.([0-9]+)/);
+if (!defined $_go1_minor) {
+    # Possible pre-release version of gccgo with "go version unknown".
+    # Derive Go minor version from from GCC major version, e.g.
+    # "go-12 env GOTOOLDIR" returns "/usr/lib/gcc/x86_64-linux-gnu/12".
+    # See https://go.dev/doc/install/gccgo#Releases
+    my ($_gcc_major) = (qx(go env GOTOOLDIR) =~ /\/([0-9]+)$/);
+    $_go1_minor = $_gcc_major * 2 - 6;
+}
 sub _go1_has_minor {
     my ($minor) = @_;
     return $_go1_minor >= $minor;
